//
//  CMCompressCore.m
//  CocoaMonkey
//
//  Created by tmkk on Sat Nov 08 2003.
//  Copyright (c) 2003 tmkk. All rights reserved.
//

#import "CMCompressCore.h"
#import "CocoaMonkeyController.h"
#import "All.h"
#import "GlobalFunctions.h"
#import "MACLib.h"
#import "aiff_process.h"

void CALLBACK progressCallback(int nPercentageDone);
const char* ErrorMessageString(int ErrorNumber);

static id myDelegate;
typedef struct {
    int    ErrorNumber;
    char*  ErrorMessage;
} ErrorMesageType;
const  ErrorMesageType  ErrorMessages[] = {
    ERROR_EXPLANATION
};

const char* ErrorMessageString(int ErrorNumber)
{
    for ( int i = 0; i < sizeof (ErrorMessages) / sizeof (*ErrorMessages); i++ )
        if ( ErrorMessages [i]. ErrorNumber == ErrorNumber )
            return  ErrorMessages[i].ErrorMessage;
    return "Unknown error";
}


@implementation CMCompressCore

- (id)initWithFiles:(NSArray *)files mode:(int)m rename:(BOOL)flag
{
	[super init];
	fileArray = [files retain];
	outputDir = nil;
	mode = m;
	compressionLevel = 2000;
	outputMode = 0;
	killFlag = 0;
	doesRenameDupName = flag;
	return self;
}

- (void)dealloc
{
	if(fileArray) [fileArray release];
	if(outputDir) [outputDir release];
	if(delegate) [delegate release];
	[super dealloc];
}

- (void)setOutputDir:(NSString *)dir
{
	outputDir = [dir retain];
}

- (void)setOutputMode:(int)m
{
	outputMode = m;
}

- (void)setCompressionLevel:(int)level
{
	compressionLevel = level;
}

- (void)setDelegate:(id)obj
{
	delegate = [obj retain];
	myDelegate = delegate;
}

- (void)startProcess
{
	if(mode == 0)
		[NSThread detachNewThreadSelector:@selector(compress) toTarget:self withObject:nil];
	else
		[NSThread detachNewThreadSelector:@selector(decompress) toTarget:self withObject:nil];
}

- (void)compress
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	int i,n,k,ret;
	int nPercentageDone;
	char *inputFile;
	NSString *outputFile, *time;
	NSFileManager *fm = [NSFileManager defaultManager];
	char tmp[4];
	FILE *fp;
	
	n = [fileArray count];
	for(i=0;i<n;i++) {
		k=1;
		NSAutoreleasePool *pool2 = [[NSAutoreleasePool alloc] init];
		inputFile = [[fileArray objectAtIndex:i] UTF8String];
		if(outputDir)
				outputFile = [outputDir stringByAppendingPathComponent:[[[[fileArray objectAtIndex:i] stringByDeletingPathExtension] stringByAppendingPathExtension:@"ape"] lastPathComponent]];
		else
				outputFile = [[[fileArray objectAtIndex:i] stringByDeletingLastPathComponent] stringByAppendingPathComponent:[[[[fileArray objectAtIndex:i] stringByDeletingPathExtension] stringByAppendingPathExtension:@"ape"] lastPathComponent]];
		
		if(doesRenameDupName) {
			if([fm fileExistsAtPath:outputFile]) {
				outputFile = [[outputFile stringByDeletingPathExtension] stringByAppendingPathExtension:[NSString stringWithFormat:@"%d.ape",k]];
				while([fm fileExistsAtPath:outputFile]) {
					k++;
					outputFile = [outputFile stringByDeletingPathExtension];
					outputFile = [[outputFile stringByDeletingPathExtension] stringByAppendingPathExtension:[NSString stringWithFormat:@"%d.ape",k]];
				}
			}
		}
		[delegate setStatus:[NSString stringWithFormat:NSLocalizedString(@"statusencoding",@""),[[fileArray objectAtIndex:i] lastPathComponent],i+1,n]];
		
		if((fp = fopen(inputFile, "rb")) == NULL) {
			time = [[NSDate date] descriptionWithCalendarFormat:@"%H:%M:%S" timeZone:nil locale:nil];
			[delegate setErrorLogText:[NSString stringWithFormat:@"[%@] encoding %@: %s\n",time,[[fileArray objectAtIndex:i] lastPathComponent],"I/O read error"]];
			[delegate setStatusImageAtRow:i mode:mode name:@"error"];
			continue;
		}
		fread(tmp, 1, 4, fp);
		fclose(fp);
		if(!strncmp(tmp,"RIFF",4))
			ret = CompressFile(inputFile, [outputFile UTF8String], compressionLevel, &nPercentageDone, progressCallback, &killFlag );
		else if(!strncmp(tmp,"MAC ",4))
			ret = ConvertFile(inputFile, [outputFile UTF8String], compressionLevel, &nPercentageDone, progressCallback, &killFlag );
		else
			ret = encodeFromAIFF(inputFile, [outputFile UTF8String], compressionLevel, delegate, &killFlag);
		if(!ret) [delegate setStatusImageAtRow:i mode:mode name:@"done"];
		else {
			time = [[NSDate date] descriptionWithCalendarFormat:@"%H:%M:%S" timeZone:nil locale:nil];
			[delegate setErrorLogText:[NSString stringWithFormat:@"[%@] encoding %@: %s\n",time,[[fileArray objectAtIndex:i] lastPathComponent],ErrorMessageString(ret)]];
			[delegate setStatusImageAtRow:i mode:mode name:@"error"];
		}
		[delegate setProgressBar:0];
		[pool2 release];
		if(killFlag) break;
	}
	[delegate setStatus:NSLocalizedString(@"idle",@"")];
	[delegate processFinished:mode];
	[pool release];
}

- (void)decompress
{
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	int i,n,k,ret;
	int nPercentageDone;
	char *inputFile;
	NSString *outputFile;
	NSFileManager *fm = [NSFileManager defaultManager];
	NSString *extension;
	
	n = [fileArray count];
	
	if(outputMode == 0) extension = @"wav";
	else if(outputMode == 1) extension = @"aif";
	else if(outputMode == 4) extension = @"ogg";
	else if(outputMode == 5) extension = @"flac";
	else extension = @"pcm";
	
	for(i=0;i<n;i++) {
		k=1;
		NSAutoreleasePool *pool2 = [[NSAutoreleasePool alloc] init];
		inputFile = [[fileArray objectAtIndex:i] UTF8String];
		if(outputDir)
				outputFile = [outputDir stringByAppendingPathComponent:[[[[fileArray objectAtIndex:i] stringByDeletingPathExtension] stringByAppendingPathExtension:extension] lastPathComponent]];
		else
				outputFile = [[[fileArray objectAtIndex:i] stringByDeletingLastPathComponent] stringByAppendingPathComponent:[[[[fileArray objectAtIndex:i] stringByDeletingPathExtension] stringByAppendingPathExtension:extension] lastPathComponent]];
		
		if(doesRenameDupName) {
			if([fm fileExistsAtPath:outputFile]) {
				outputFile = [[outputFile stringByDeletingPathExtension] stringByAppendingPathExtension:[NSString stringWithFormat:@"%d.%@",k,extension]];
				while([fm fileExistsAtPath:outputFile]) {
					k++;
					outputFile = [outputFile stringByDeletingPathExtension];
					outputFile = [[outputFile stringByDeletingPathExtension] stringByAppendingPathExtension:[NSString stringWithFormat:@"%d.%@",k,extension]];
				}
			}
		}
		
		[delegate setStatus:[NSString stringWithFormat:NSLocalizedString(@"statusdecoding",@""),[[fileArray objectAtIndex:i] lastPathComponent],i+1,n]];
		if(!strcasecmp([[[fileArray objectAtIndex:i] pathExtension] UTF8String],"cue")) {
			if(outputMode == 4) ret = decodeApeWithCueAndCompress(inputFile, [[outputFile stringByDeletingLastPathComponent] UTF8String], delegate, &killFlag, outputMode,doesRenameDupName,[delegate getVorbisQuality]);
			else if(outputMode == 5) ret = decodeApeWithCueAndCompress(inputFile, [[outputFile stringByDeletingLastPathComponent] UTF8String], delegate, &killFlag, outputMode,doesRenameDupName,(float)[delegate getFLACCompressionLevel]);
			else ret = decodeApeWithCue(inputFile, [[outputFile stringByDeletingLastPathComponent] UTF8String], delegate, &killFlag, outputMode,doesRenameDupName);
		}
		else if(outputMode == 0) //WAVE
			ret = DecompressFile (inputFile, [outputFile UTF8String], &nPercentageDone, progressCallback, &killFlag );
		else if(outputMode == 1) //AIFF
			ret = decodeToAIFF(inputFile, [outputFile UTF8String], delegate, &killFlag);
		else if(outputMode == 4) //Vorbis
			ret = decodeToVorbis(inputFile, [outputFile UTF8String], delegate, &killFlag, [delegate getVorbisQuality]);
		else if(outputMode == 5) //FLAC
			ret = decodeToFLAC(inputFile, [outputFile UTF8String], delegate, &killFlag, [delegate getFLACCompressionLevel]);
		else //PCM
			ret = decodeToPCM(inputFile, [outputFile UTF8String], delegate, &killFlag, outputMode);
		if(!ret) [delegate setStatusImageAtRow:i mode:mode name:@"done"];
		else {
			NSString *time = [[NSDate date] descriptionWithCalendarFormat:@"%H:%M:%S" timeZone:nil locale:nil];
			[delegate setErrorLogText:[NSString stringWithFormat:@"[%@] decoding %@: %s\n",time,[[fileArray objectAtIndex:i] lastPathComponent],ErrorMessageString(ret)]];
			[delegate setStatusImageAtRow:i mode:mode name:@"error"];
		}
		[delegate setProgressBar:0];
		[pool2 release];
		if(killFlag) break;
	}
	[delegate setStatus:NSLocalizedString(@"idle",@"")];
	[delegate processFinished:mode];
	[pool release];
}

- (void)stopProcess
{
	killFlag = 1;
}

void CALLBACK progressCallback(int nPercentageDone)
{
    [myDelegate setProgressBar:nPercentageDone/1000];
}

@end
