#import "CocoaMonkeyController.h"
#import "CMCompressCore.h"

@implementation CocoaMonkeyController

- (id)init
{
	[super init];
	compressFiles = [[NSMutableArray alloc] init];
	decompressFiles = [[NSMutableArray alloc] init];
	encodeStatusArray = [[NSMutableArray alloc] init];
	decodeStatusArray = [[NSMutableArray alloc] init];
	encodeFinished = NO;
	decodeFinished = NO;
	isEncoding = NO;
	isDecoding = NO;
	doesShowFullPath = YES;
	return self;
}

- (IBAction)startDecode:(id)sender
{
	BOOL renameFlag;
	if(isDecoding) {
		[compressCore stopProcess];
		return;
	}
	isDecoding = YES;
	[decodeButton setTitle:NSLocalizedString(@"stop",@"")];
	[encodeButton setEnabled:NO];
	[decodeStatusArray removeAllObjects];
	[decompressFileTable reloadData];
	if([[dupRadio selectedCell] tag] == 0) renameFlag = YES;
	else renameFlag = NO;
	compressCore = [[CMCompressCore alloc] initWithFiles:decompressFiles mode:1 rename:renameFlag];
	[compressCore setDelegate:self];
	[compressCore setOutputMode:[[outputMode selectedItem] tag]];
	if([[outputRadio selectedCell] tag] == 1) [compressCore setOutputDir:[fixedOutputDir stringValue]];
	[compressCore startProcess];
}

- (IBAction)startEncode:(id)sender
{
	BOOL renameFlag;
	if(isEncoding) {
		[compressCore stopProcess];
		return;
	}
	isEncoding = YES;
	[encodeButton setTitle:NSLocalizedString(@"stop",@"")];
	[decodeButton setEnabled:NO];
	[encodeStatusArray removeAllObjects];
	[compressFileTable reloadData];
	if([[dupRadio selectedCell] tag] == 0) renameFlag = YES;
	else renameFlag = NO;
	compressCore = [[CMCompressCore alloc] initWithFiles:compressFiles mode:0 rename:renameFlag];
	[compressCore setDelegate:self];
	[compressCore setCompressionLevel:[[compressionLevel selectedItem] tag]];
	if([[outputRadio selectedCell] tag] == 1)
		[compressCore setOutputDir:[fixedOutputDir stringValue]];
	[compressCore startProcess];
}

- (IBAction)prefOK:(id)sender
{
	[prefPane close];
	[self savePrefs];
	if([pathStyle state] == NSOnState) doesShowFullPath = YES;
	else doesShowFullPath = NO;
	[compressFileTable reloadData];
	[decompressFileTable reloadData];
}

- (IBAction)prefCancel:(id)sender
{
	[prefPane close];
	[self loadPrefs];
}

- (IBAction)showPref:(id)sender
{
	[prefPane center];
	[prefPane makeKeyAndOrderFront:self];
}

- (IBAction)enableOutputDir:(id)sender
{
	if([[sender selectedCell] tag] == 1) {
		[fixedOutputDir setEnabled: YES];
		[setOutputDirButton setEnabled:YES];
	}
	else {
		[fixedOutputDir setEnabled: NO];
		[setOutputDirButton setEnabled:NO];
	}
}

- (IBAction)setOutputDir:(id)sender
{
	NSOpenPanel *op;
	int stat;
	
	op = [ NSOpenPanel openPanel ];
	
	//[ op setTitle : NSLocalizedString(@"selectdir",@"") ];
	[ op setCanChooseDirectories : YES ];
	[ op setCanChooseFiles : NO ];
	
	stat = [ op runModalForDirectory : nil
								file : nil
							   types : nil ];
							   
	if(stat != NSOKButton) return;
	
	[fixedOutputDir setStringValue:[op filename]];
}

- (IBAction)showOption:(id)sender
{
	if([[outputMode selectedItem] tag] == 4) {
		[NSApp beginSheet: vorbisQualityPanel
			modalForWindow: prefPane
			modalDelegate: self
			didEndSelector: @selector(sheetDidEnd:returnCode:contextInfo:)
			contextInfo: nil];
	}
	else if([[outputMode selectedItem] tag] == 5) {
		[NSApp beginSheet: flacOptionPanel
			modalForWindow: prefPane
			modalDelegate: self
			didEndSelector: @selector(sheetDidEnd:returnCode:contextInfo:)
			contextInfo: nil];
	}
}

- (IBAction)closeOption:(id)sender
{
	[NSApp endSheet: [NSApp keyWindow] returnCode: 1];
}

- (IBAction)toggleOptionButton:(id)sender
{
	if(hasVorbisPlugin || hasFLACPlugin) {
		if([[sender selectedItem] tag] == 4 || [[sender selectedItem] tag] == 5) {
			[decoderOptionButton setEnabled: YES];
		}
		else {
			[decoderOptionButton setEnabled: NO];
		}
	}
}

- (float)getVorbisQuality
{
	return [qualityValue floatValue] / 10.0f;
}

- (int)getFLACCompressionLevel
{
	return [flacCompressionLevel intValue];
}

- (void)sheetDidEnd:(NSWindow *)sheet
			returnCode:(int)returnCode
			contextInfo:(void *)contextInfo
{
	
	[sheet close];
	[prefPane makeKeyAndOrderFront: self];
}


- (IBAction)clearEncodeTable:(id)sender
{
	if(isEncoding) return;
	encodeFinished = NO;
	[compressFiles removeAllObjects];
	[encodeStatusArray removeAllObjects];
	[compressFileTable reloadData];
}

- (IBAction)clearDecodeTable:(id)sender
{
	if(isDecoding) return;
	decodeFinished = NO;
	[decompressFiles removeAllObjects];
	[decodeStatusArray removeAllObjects];
	[decompressFileTable reloadData];
}

- (IBAction)addEncodeItem:(id)sender
{
	NSOpenPanel *op;
	int stat;
	
	op = [ NSOpenPanel openPanel ];
	
	//[ op setTitle : NSLocalizedString(@"selectdir",@"") ];
	[ op setCanChooseDirectories : NO ];
	[ op setCanChooseFiles : YES ];
	[op setAllowsMultipleSelection: YES];
	
	stat = [ op runModalForDirectory : nil
								file : nil
							   types : nil ];
							   
	if(stat != NSOKButton) return;
	
	[self appendCompressFile: [op filenames]];
}

- (IBAction)addDecodeItem:(id)sender
{
	NSOpenPanel *op;
	int stat;
	
	op = [ NSOpenPanel openPanel ];
	
	//[ op setTitle : NSLocalizedString(@"selectdir",@"") ];
	[ op setCanChooseDirectories : NO ];
	[ op setCanChooseFiles : YES ];
	[op setAllowsMultipleSelection: YES];
	
	stat = [ op runModalForDirectory : nil
								file : nil
							   types : nil ];
							   
	if(stat != NSOKButton) return;
	
	[self appendDecompressFile: [op filenames]];
}

- (IBAction)showErrorLog:(id)sender
{
	[errorLogPane makeKeyAndOrderFront:self];
	//[showErrorLogMenu setEnabled:NO];
}

- (void)setProgressBar:(int)percentage
{
	[progressIndicator setDoubleValue:(double)percentage];
}

- (void)appendCompressFile:(NSArray *)files
{
	if(isEncoding) return;
	if(encodeFinished) {
		[compressFiles removeAllObjects];
		[encodeStatusArray removeAllObjects];
		encodeFinished = NO;
	}
	[compressFiles addObjectsFromArray:files];
	[compressFileTable reloadData];
}

- (void)appendDecompressFile:(NSArray *)files
{
	if(isDecoding) return;
	if(decodeFinished) {
		[decompressFiles removeAllObjects];
		[decodeStatusArray removeAllObjects];
		decodeFinished = NO;
	}
	[decompressFiles addObjectsFromArray:files];
	[decompressFileTable reloadData];
}

- (void)setStatus:(NSString *)str
{
	id superview;
	superview = [statusText superview];
	[statusText removeFromSuperview];
	[statusText setStringValue:@""];
	[statusText setStringValue:str];
	[superview addSubview:statusText];
	
}

- (void)loadPrefs
{
	NSUserDefaults *pref = [NSUserDefaults standardUserDefaults];
	id temp;
	int i;
	if(temp=[pref objectForKey:@"CompressionLevel"]) {
		i = [temp intValue];
		if(i < [compressionLevel numberOfItems]) [compressionLevel selectItemAtIndex: i];
	}
	if(temp=[pref objectForKey:@"DupRadio"]) {
		i = [temp intValue];
		[dupRadio selectCellWithTag: i];
	}
	if(temp=[pref objectForKey:@"OutputRadio"]) {
		i = [temp intValue];
		[outputRadio selectCellWithTag: i];
		if(i==1) {
			[fixedOutputDir setEnabled: YES];
			[setOutputDirButton setEnabled:YES];
		}
		else {
			[fixedOutputDir setEnabled: NO];
			[setOutputDirButton setEnabled:NO];
		}
	}
	if(temp=[pref objectForKey:@"OutputDir"]) {
		[fixedOutputDir setStringValue:temp];
	}
	else [fixedOutputDir setStringValue:[@"~/Desktop" stringByExpandingTildeInPath]];
	if(temp=[pref objectForKey:@"PathStyle"]) {
		i = [temp intValue];
		[pathStyle setState: i];
	}
	if(temp=[pref objectForKey:@"OutputMode"]) {
		i = [temp intValue];
		if(i < [outputMode numberOfItems]) [outputMode selectItemAtIndex: i];
		if(i==4 || i==5) [decoderOptionButton setEnabled: YES];
		else [decoderOptionButton setEnabled: NO];
	}
	if(temp=[pref objectForKey:@"VorbisQuality"]) {
		[qualityValue setFloatValue: [temp floatValue]];
		[vorbisQualitySlider setFloatValue: [temp floatValue]];
	}
	if(temp=[pref objectForKey:@"FLACCompressionLevel"]) {
		[flacCompressionLevel setIntValue: [temp intValue]];
	}
}

- (void)savePrefs
{
	NSUserDefaults *pref = [NSUserDefaults standardUserDefaults];
	[pref setInteger:[compressionLevel indexOfSelectedItem] forKey:@"CompressionLevel"];
	[pref setObject:[fixedOutputDir stringValue] forKey:@"OutputDir"];
	[pref setInteger:[[dupRadio selectedCell] tag] forKey:@"DupRadio"];
	[pref setInteger:[[outputRadio selectedCell] tag] forKey:@"OutputRadio"];
	[pref setInteger:[pathStyle state] forKey:@"PathStyle"];
	[pref setInteger:[outputMode indexOfSelectedItem] forKey:@"OutputMode"];
	[pref setFloat:[qualityValue floatValue] forKey:@"VorbisQuality"];
	[pref setInteger:[flacCompressionLevel intValue] forKey:@"FLACCompressionLevel"];
	[pref synchronize];
}

- (void)setStatusImageAtRow:(int)row mode:(int)m name:(NSString *)name
{
	if(m==0) {
		[encodeStatusArray addObject:[NSImage imageNamed:name]];
		[compressFileTable reloadData];
	}
	else {
		[decodeStatusArray addObject:[NSImage imageNamed:name]];
		[decompressFileTable reloadData];
	}
}

- (void)processFinished:(int)m
{
	if(m==0) {
		[decodeButton setEnabled:YES];
		[encodeButton setTitle:NSLocalizedString(@"encode",@"")];
		encodeFinished = YES;
		isEncoding = NO;
	}
	else {
		[encodeButton setEnabled:YES];
		[decodeButton setTitle:NSLocalizedString(@"decode",@"")];
		decodeFinished = YES;
		isDecoding = NO;
	}
	[compressCore release];
}

- (void)removeSelectedFile:(id)table
{
	int n,i;
	id statusArray,fileArray;
	BOOL isProcessing,processFinished;
	if([table tag] == 0) {
		statusArray = encodeStatusArray;
		fileArray = compressFiles;
		isProcessing = isEncoding;
		processFinished = encodeFinished;
	}
	else {
		statusArray = decodeStatusArray;
		fileArray = decompressFiles;
		isProcessing = isDecoding;
		processFinished = decodeFinished;
	}
	if(![fileArray count] || isProcessing) return;
	n = [table numberOfRows];
	for(i=n-1;i>=0;i--) {
		if([table isRowSelected:i]) {
			[fileArray removeObjectAtIndex:i];
			if(i < [statusArray count]) [statusArray removeObjectAtIndex:i];
			processFinished = NO;
		}
	}
	[table reloadData];
}

- (void)removeSelectedDecompressFile
{
	int row;
	if(![decompressFiles count] || isDecoding) return;
	row = [decompressFileTable selectedRow];
	[decompressFiles removeObjectAtIndex:row];
	if(row < [decodeStatusArray count]) [decodeStatusArray removeObjectAtIndex:row];
	decodeFinished = NO;
	[decompressFileTable reloadData];
}

- (void)setErrorLogText:(NSString *)str
{
	NSAttributedString *attribStr;
	NSMutableDictionary *tmp = [[NSMutableDictionary alloc] init];
	[tmp setObject:[NSFont systemFontOfSize:11] forKey: NSFontAttributeName];
	attribStr = [[NSAttributedString alloc] initWithString: str attributes: tmp];
	[[errorLog textStorage] appendAttributedString:attribStr];
	[tmp release];
	[attribStr release];
}

/* delegate methods */

- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
	if([tableView tag] == 0)
		return [compressFiles count];
	else
		return [decompressFiles count];
}

- (id)tableView:(NSTableView *)tableView
		objectValueForTableColumn:(NSTableColumn *)tableColumn
		row:(int)row
{
	id identifier = [tableColumn identifier];
	if([tableView tag] == 0) {
		if([identifier isEqual:@"Status"])  {
			if(row < [encodeStatusArray count]) return [encodeStatusArray objectAtIndex:row];
			else return nil;
		}
		else {
			if(doesShowFullPath) return [compressFiles objectAtIndex:row];
			else return [[compressFiles objectAtIndex:row] lastPathComponent];
		}
	}
	else {
		if([identifier isEqual:@"Status"]) {
			if(row < [decodeStatusArray count]) return [decodeStatusArray objectAtIndex:row];
			else return nil;
		}
		else {
			if(doesShowFullPath) return [decompressFiles objectAtIndex:row];
			else return [[decompressFiles objectAtIndex:row] lastPathComponent];
		}
	}
}

- (void)applicationDidFinishLaunching: (NSNotification *)notification
{
	[self loadPrefs];
	NSArray* array = [NSArray arrayWithObject:NSFilenamesPboardType];
	[compressFileTable registerForDraggedTypes:array];
	[decompressFileTable registerForDraggedTypes:array];
	[[compressFileTable tableColumnWithIdentifier:@"Status"] setDataCell:[[NSImageCell alloc]init]];
	[[decompressFileTable tableColumnWithIdentifier:@"Status"] setDataCell:[[NSImageCell alloc]init]];
	[mainWindow setFrameAutosaveName: @"MainWindow"];
	[mainWindow makeKeyAndOrderFront: self];
	if([pathStyle state] == NSOnState) doesShowFullPath = YES;
	else doesShowFullPath = NO;
	//vorbisPlugin = [[NSBundle alloc] initWithPath: [[[[NSBundle mainBundle] bundlePath] stringByDeletingLastPathComponent] stringByAppendingPathComponent: @"CMVorbisPlugin.bundle"]];
	vorbisPlugin = [[NSBundle alloc] initWithPath: [[NSBundle mainBundle] pathForResource:@"CMVorbisPlugin" ofType:@"bundle" inDirectory:nil]];
	if(!vorbisPlugin) {
		hasVorbisPlugin = NO;
	}
	else {
		hasVorbisPlugin = YES;
		vorbisEncoderClass = [vorbisPlugin classNamed:@"CMVorbisEnc"];
		if(!vorbisEncoderClass) {
			hasVorbisPlugin = NO;
		}
	}
	if(!hasVorbisPlugin) {
		[decoderOptionButton setEnabled: NO];
		if([[outputMode titleOfSelectedItem] isEqualToString: @"Ogg Vorbis"]) {
			[decoderOptionButton setEnabled: NO];
			[outputMode selectItemAtIndex: 0];
		}
		[outputMode removeItemWithTitle:@"Ogg Vorbis"];
		[self savePrefs];
	}
	flacPlugin = [[NSBundle alloc] initWithPath: [[NSBundle mainBundle] pathForResource:@"CMFLACPlugin" ofType:@"bundle" inDirectory:nil]];
	if(!flacPlugin) {
		hasFLACPlugin = NO;
	}
	else {
		hasFLACPlugin = YES;
		FLACEncoderClass = [flacPlugin classNamed:@"CMFLACEnc"];
		if(!FLACEncoderClass) {
			hasFLACPlugin = NO;
		}
	}
	if(!hasFLACPlugin) {
		if([[outputMode titleOfSelectedItem] isEqualToString: @"FLAC"]) {
			[decoderOptionButton setEnabled: NO];
			[outputMode selectItemAtIndex: 0];
		}
		[outputMode removeItemWithTitle:@"FLAC"];
		[self savePrefs];
	}
	
}

- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
	if(isEncoding || isDecoding) {
		int ret = NSRunAlertPanel(NSLocalizedString(@"quit",@""), NSLocalizedString(@"quitnotice",@""),NSLocalizedString(@"cancel",@""), nil, @"OK");
		if(ret == NSAlertDefaultReturn) return NO;
	}
	[self savePrefs];
	return YES;
}

- (BOOL)windowShouldClose:(id)sender
{
	[NSApp terminate:nil];
	return NO;
}

- (Class)vorbisEncoderClass
{
	return vorbisEncoderClass;
}

- (Class)FLACEncoderClass
{
	return FLACEncoderClass;
}

@end
